@@ -183,6 +183,7 @@ impl TTYDecoder {
             Box::new(DeviceAttrsMatcher),
             Box::new(GraphicRenditionMatcher::default()),
             Box::new(KittyImageMatcher),
+            Box::new(KittyKeyboardMatcher),
             Box::new(MouseEventMatcher),
             Box::new(OSControlMatcher),
             Box::new(ReportSettingMatcher),
@@ -327,6 +328,83 @@ impl TTYMatcher for KittyImageMatcher {
     }
 }
 
+/// Kitty Keyboard Protocol
+///
+/// Reference: https://sw.kovidgoyal.net/kitty/keyboard-protocol
+#[derive(Debug)]
+struct KittyKeyboardMatcher;
+
+pub(crate) const KEYBOARD_LEVEL: usize = 1;
+
+impl TTYMatcher for KittyKeyboardMatcher {
+    fn matcher(&self) -> NFA<_Void> {
+        NFA::sequence([
+            NFA::from("\x1b["),
+            NFA::choice([
+                NFA::from("?") + NFA::digit().some(),
+                NFA::predicate(|c| matches!(c, b';' | b':' | b'0'..=b'9')).many(),
+            ]),
+            NFA::from("u"),
+        ])
+    }
+
+    fn decode(&mut self, data: &[u8]) -> Option<TerminalEvent> {
+        let data = &data[2..data.len() - 1]; // skip CSI and `u`
+        if data[0] == b'?' {
+            let level = number_decode(&data[1..data.len()])?;
+            return Some(TerminalEvent::KeyboardLevel(level));
+        }
+
+        let mut fields = data.split(|c| *c == b';');
+
+        // decode key
+        let name = match fields.next() {
+            Some(codes) => {
+                // TODO: decode alternative keys
+                let mut codes = numbers_decode(codes, b':');
+                keyboard_decode_key(codes.next().unwrap_or(1))?
+            }
+            None => return None,
+        };
+
+        // decode modifiers
+        let mode = match fields.next() {
+            Some(modes) => {
+                let mut modes = numbers_decode(modes, b':');
+                let mode = match modes.next() {
+                    Some(mode) if mode > 1 => KeyMod::from_bits((mode - 1) as u32),
+                    _ => KeyMod::EMPTY,
+                };
+                let event_type = modes.next().unwrap_or(0);
+                // TODO: decode press/release/repeat
+                if event_type != 0 {
+                    return None;
+                }
+                mode
+            }
+            None => KeyMod::EMPTY,
+        };
+
+        // TODO: decode text as codepoint
+        let _text = fields.next();
+
+        Some(TerminalEvent::Key(Key { name, mode }))
+    }
+}
+
+fn keyboard_decode_key(code: usize) -> Option<KeyName> {
+    let key = match code {
+        27 => KeyName::Esc,
+        13 => KeyName::Enter,
+        9 => KeyName::Tab,
+        127 => KeyName::Backspace,
+        code @ 57376..=57398 => KeyName::F(code - 57376 + 13),
+        code if code <= u32::MAX as usize => KeyName::Char(char::from_u32(code as u32)?),
+        _ => return None,
+    };
+    Some(key)
+}
+
 /// DECRPM - DEC mode report
 ///
 /// Rerference: https://www.vt100.net/docs/vt510-rm/DECRPM
@@ -346,7 +424,7 @@ impl TTYMatcher for DecModeMatcher {
 
     fn decode(&mut self, data: &[u8]) -> Option<TerminalEvent> {
         // "\x1b[?{mode};{status}$y"
-        let mut nums = numbers_decode(&data[3..data.len() - 2]);
+        let mut nums = numbers_decode(&data[3..data.len() - 2], b';');
         Some(TerminalEvent::DecMode {
             mode: crate::terminal::DecMode::from_usize(nums.next()?)?,
             status: DecModeStatus::from_usize(nums.next()?)?,
@@ -372,7 +450,7 @@ impl TTYMatcher for DeviceAttrsMatcher {
 
     fn decode(&mut self, data: &[u8]) -> Option<TerminalEvent> {
         Some(TerminalEvent::DeviceAttrs(
-            numbers_decode(&data[3..data.len() - 1])
+            numbers_decode(&data[3..data.len() - 1], b';')
                 .filter(|v| v > &0)
                 .collect(),
         ))
@@ -499,12 +577,12 @@ impl TTYMatcher for MouseEventMatcher {
 
     fn decode(&mut self, data: &[u8]) -> Option<TerminalEvent> {
         // "\x1b[<{event};{row};{col}(m|M)"
-        let mut nums = numbers_decode(&data[3..data.len() - 1]);
+        let mut nums = numbers_decode(&data[3..data.len() - 1], b';');
         let event = nums.next()?;
         let col = nums.next()? - 1;
         let row = nums.next()? - 1;
 
-        let mut mode = KeyMod::from_bits(((event >> 2) & 7) as u8);
+        let mut mode = KeyMod::from_bits(((event >> 2) & 7) as u32);
         if data[data.len() - 1] == b'M' {
             mode |= KeyMod::PRESS;
         }
@@ -634,7 +712,7 @@ impl TTYMatcher for CursorPositionMatcher {
 
     fn decode(&mut self, data: &[u8]) -> Option<TerminalEvent> {
         // "\x1b[{row};{col}R"
-        let mut nums = numbers_decode(&data[2..data.len() - 1]);
+        let mut nums = numbers_decode(&data[2..data.len() - 1], b';');
         Some(TerminalEvent::CursorPosition {
             row: nums.next()? - 1,
             col: nums.next()? - 1,
@@ -670,11 +748,11 @@ impl TTYMatcher for TermSizeMatcher {
         let mut chunks = data.split(|c| *c == b'\x1b');
         chunks.next()?; // empty
         let cell_size = chunks.next()?;
-        let mut nums = numbers_decode(&cell_size[3..cell_size.len() - 1]);
+        let mut nums = numbers_decode(&cell_size[3..cell_size.len() - 1], b';');
         let cell_height = nums.next()?;
         let cell_width = nums.next()?;
         let pixel_size = chunks.next()?;
-        let mut nums = numbers_decode(&pixel_size[3..pixel_size.len() - 1]);
+        let mut nums = numbers_decode(&pixel_size[3..pixel_size.len() - 1], b';');
         let pixel_height = nums.next()?;
         let pixel_width = nums.next()?;
         Some(TerminalEvent::Size(TerminalSize {
@@ -743,14 +821,14 @@ fn tty_event_nfa() -> NFA<TerminalEvent> {
         (KeyName::End, "4"),
         (KeyName::PageUp, "5"),
         (KeyName::PageDown, "6"),
-        (KeyName::F5, "15"),
-        (KeyName::F6, "17"),
-        (KeyName::F7, "18"),
-        (KeyName::F8, "19"),
-        (KeyName::F9, "20"),
-        (KeyName::F10, "21"),
-        (KeyName::F11, "23"),
-        (KeyName::F12, "24"),
+        (KeyName::F(5), "15"),
+        (KeyName::F(6), "17"),
+        (KeyName::F(7), "18"),
+        (KeyName::F(8), "19"),
+        (KeyName::F(9), "20"),
+        (KeyName::F(10), "21"),
+        (KeyName::F(11), "23"),
+        (KeyName::F(12), "24"),
     ]
     .iter()
     {
@@ -769,10 +847,10 @@ fn tty_event_nfa() -> NFA<TerminalEvent> {
         (KeyName::Left, "[", "D"),
         (KeyName::End, "[", "F"),
         (KeyName::Home, "[", "H"),
-        (KeyName::F1, "O", "P"),
-        (KeyName::F2, "O", "Q"),
-        (KeyName::F3, "O", "R"),
-        (KeyName::F4, "O", "S"),
+        (KeyName::F(1), "O", "P"),
+        (KeyName::F(2), "O", "Q"),
+        (KeyName::F(3), "O", "R"),
+        (KeyName::F(4), "O", "S"),
     ]
     .iter()
     {
@@ -889,8 +967,8 @@ fn key_value_decode(sep: u8, data: &[u8]) -> impl Iterator<Item = (&[u8], &[u8])
 }
 
 /// Semi-colon separated positve numers
-fn numbers_decode(data: &[u8]) -> impl Iterator<Item = usize> + '_ {
-    data.split(|b| *b == b';').filter_map(number_decode)
+fn numbers_decode(data: &[u8], sep: u8) -> impl Iterator<Item = usize> + '_ {
+    data.split(move |b| *b == sep).filter_map(number_decode)
 }
 
 // Decode positive integer number
@@ -965,13 +1043,13 @@ mod tests {
 
         assert_eq!(
             decoder.decode(&mut cursor)?,
-            Some(TerminalEvent::Key(KeyName::F3.into()))
+            Some(TerminalEvent::Key(KeyName::F(3).into()))
         );
         assert_eq!(cursor.position(), 3);
 
         assert_eq!(
             decoder.decode(&mut cursor)?,
-            Some(TerminalEvent::Key(KeyName::F5.into()))
+            Some(TerminalEvent::Key(KeyName::F(5).into()))
         );
         assert_eq!(cursor.position(), 3 + 5);
 
@@ -1130,12 +1208,12 @@ mod tests {
             }),
         );
 
-        write!(cursor.get_mut(), "\x1b[?2017;0$y")?;
+        write!(cursor.get_mut(), "\x1b[?2026;0$y")?;
 
         assert_eq!(
             decoder.decode(&mut cursor)?,
             Some(TerminalEvent::DecMode {
-                mode: DecMode::KittyKeyboard,
+                mode: DecMode::SynchronizedOutput,
                 status: DecModeStatus::NotRecognized,
             }),
         );
@@ -1365,4 +1443,30 @@ mod tests {
 
         Ok(())
     }
+
+    #[test]
+    fn test_kitty_keyboard() -> Result<(), Error> {
+        let mut cursor = Cursor::new(Vec::new());
+        let mut decoder = TTYDecoder::new();
+
+        write!(cursor.get_mut(), "\x1b[?15u")?;
+        write!(cursor.get_mut(), "\x1b[27;7u")?;
+        write!(cursor.get_mut(), "\x1b[99;5u")?;
+        write!(cursor.get_mut(), "\x1b[1;6P")?;
+
+        let mut result = Vec::new();
+        decoder.decode_into(&mut cursor, &mut result)?;
+
+        assert_eq!(
+            result,
+            vec![
+                TerminalEvent::KeyboardLevel(15),
+                TerminalEvent::Key("ctrl+alt+esc".parse()?),
+                TerminalEvent::Key("ctrl+c".parse()?),
+                TerminalEvent::Key("ctrl+shift+f1".parse()?),
+            ],
+        );
+
+        Ok(())
+    }
 }
